require 'tennpipes-base/cli/base' unless defined?(Tennpipes::Cli::Base)
require 'net/https'

module Tennpipes
  module Generators
    ##
    # Responsible for executing plugins instructions within a Tennpipes project.
    #
    class Plugin < Thor::Group
      # Defines the default URL for official tennpipes recipe plugins.
      PLUGIN_URL = 'https://github.com/tennpipes/tennpipes-recipes/tree/master/plugins'
      Tennpipes::Generators.add_generator(:plugin, self)

      def self.source_root; File.expand_path(File.dirname(__FILE__)); end
      def self.banner; "tennpipes-init plugin [plugin_identifier] [options]"; end

      include Thor::Actions
      include Tennpipes::Generators::Actions
      include Tennpipes::Generators::Runner

      desc "Description:\n\n\ttennpipes-init plugin sets up a plugin within a Tennpipes application"

      argument :plugin_file, :desc => "The name or path to the Tennpipes plugin", :optional => true

      class_option :root, :desc => "The root destination", :aliases => '-r', :default => ".",   :type => :string
      class_option :list, :desc => "list available plugins", :aliases => '-l', :default => false, :type => :boolean
      class_option :destroy, :aliases => '-d', :default => false, :type => :boolean

      # Show help if no ARGV given.
      require_arguments!

      ##
      # Create the Tennpipes Plugin.
      #
      def setup_plugin
        if options[:list] || plugin_file.nil?
          list_plugins
        else # executing the plugin instructions
          self.destination_root = options[:root]
          if in_app_root?
            self.behavior = :revoke if options[:destroy]
            execute_runner(:plugin, plugin_file)
          else
            say "You are not at the root of a Tennpipes application! (config/boot.rb not found)"
          end
        end
      end

      no_tasks do
        def list_plugins
          plugins = {}
          uri = URI.parse(PLUGIN_URL)
          http = Net::HTTP.new(uri.host, uri.port)
          http.use_ssl = true if uri.scheme == "https"
          http.verify_mode = OpenSSL::SSL::VERIFY_NONE
          http.start do
            http.request_get(uri.path) do |res|
              plugins = res.body.scan(%r{/plugins/(\w+)_plugin.rb}).uniq
            end
          end
          say "Available plugins:", :green
          say plugins.map { |plugin| "  - #{plugin}" }.join("\n")
        end
      end
    end
  end
end
